@@ -47,6 +47,15 @@ class NetmikoSwitchTestBase(fixtures.TestWithFixtures):
 
 class TestNetmikoSwitch(NetmikoSwitchTestBase):
 
+    def test_batch(self):
+        self.cfg.config(backend_url='url', group='ngs_coordination')
+        self._make_switch_device({'ngs_batch_requests': True})
+
+    def test_batch_missing_backend_url(self):
+        self.assertRaisesRegex(
+            Exception, "backend_url",
+            self._make_switch_device, {'ngs_batch_requests': True})
+
     @mock.patch('networking_generic_switch.devices.netmiko_devices.'
                 'NetmikoSwitch.send_commands_to_device',
                 return_value='fake output')
@@ -177,20 +186,6 @@ class TestNetmikoSwitch(NetmikoSwitchTestBase):
         m_sctd.assert_called_with([])
         m_check.assert_called_once_with('fake output', 'unplug port')
 
-    @mock.patch('networking_generic_switch.devices.netmiko_devices.'
-                'NetmikoSwitch.plug_port_to_network',
-                return_value='fake output')
-    def test_plug_bond_to_network_fallback(self, m_plug):
-        self.switch.plug_bond_to_network(2222, 22)
-        m_plug.assert_called_with(2222, 22)
-
-    @mock.patch('networking_generic_switch.devices.netmiko_devices.'
-                'NetmikoSwitch.delete_port',
-                return_value='fake output')
-    def test_unplug_bond_from_network_fallback(self, m_delete):
-        self.switch.unplug_bond_from_network(2222, 22)
-        m_delete.assert_called_with(2222, 22)
-
     def test__format_commands(self):
         self.switch._format_commands(
             netmiko_devices.NetmikoSwitch.ADD_NETWORK,
@@ -270,6 +265,21 @@ class TestNetmikoSwitch(NetmikoSwitchTestBase):
         self.assertEqual('fake output', result)
         save_mock.assert_called_once_with(connect_mock)
 
+    @mock.patch.object(netmiko_devices.NetmikoSwitch, '_get_connection')
+    @mock.patch.object(netmiko_devices.NetmikoSwitch, 'send_config_set')
+    @mock.patch.object(netmiko_devices.NetmikoSwitch, 'save_configuration')
+    def test_send_commands_to_device_without_save_config(self, save_mock,
+                                                         send_mock, gc_mock):
+        switch = self._make_switch_device(
+            extra_cfg={'ngs_save_configuration': False})
+        connect_mock = mock.MagicMock(netmiko.base_connection.BaseConnection)
+        gc_mock.return_value.__enter__.return_value = connect_mock
+        send_mock.return_value = 'fake output'
+        result = switch.send_commands_to_device(['spam ham aaaa'])
+        send_mock.assert_called_once_with(connect_mock, ['spam ham aaaa'])
+        self.assertEqual('fake output', result)
+        save_mock.assert_not_called()
+
     def test_send_config_set(self):
         connect_mock = mock.MagicMock(netmiko.base_connection.BaseConnection)
         connect_mock.send_config_set.return_value = 'fake output'
